@@ -1,4 +1,3 @@
-use borsh::{BorshSerialize, BorshDeserialize};
 use quick_protobuf::message::MessageWrite;
 use quick_protobuf::writer::serialize_into_vec;
 use solana_program::pubkey::Pubkey;
@@ -7,40 +6,60 @@ use solana_program::sysvar::rent;
 use solana_program::{account_info::AccountInfo, msg, program_error::ProgramError};
 use std::clone::Clone;
 use switchboard_protos::protos::aggregator_state::AggregatorState;
-use switchboard_protos::protos::aggregator_state::RoundResult;
 use switchboard_protos::protos::aggregator_state::mod_AggregatorState;
 use switchboard_protos::protos::fulfillment_manager::FulfillmentManagerState;
 use switchboard_protos::protos::fulfillment_manager::mod_FulfillmentManagerState;
 use switchboard_protos::protos::switchboard_account_types::SwitchboardAccountType;
-use std::convert::TryInto;
+use zerocopy::{AsBytes, FromBytes, LayoutVerified};
 
-pub mod flat_aggregator_result_generated;
-pub use flat_aggregator_result_generated::*;
 pub type PubkeyBuffer = [u8; 32];
 
-#[derive(BorshSerialize, BorshDeserialize, Debug, Default, PartialEq, Clone)]
-pub struct AggregatorResultParseOptimized {
+#[repr(C)]
+#[derive(Default,AsBytes,FromBytes,Clone,Debug)]
+pub struct FastRoundResult {
+    pub num_success: i32,
+    pub num_error: i32,
+    pub result: f64,
+    pub round_open_slot: u64,
+    pub round_open_timestamp: i64,
+    pub min_response: f64,
+    pub max_response: f64,
+}
+
+#[repr(C)]
+#[derive(Default,AsBytes,FromBytes,Clone,Debug)]
+pub struct FastRoundResultAccountData {
     pub parent: PubkeyBuffer,
-    pub result: Option<RoundResult>,
+    pub result: FastRoundResult,
 }
 
-pub fn borsh_safe_serialize<T: BorshSerialize>(state: &T, type_indicator: SwitchboardAccountType) -> Vec<u8> {
-    msg!("borsh serialize");
-    let mut bytes = state.try_to_vec().unwrap();
-    let size: usize = bytes.len();
-    let mut res = vec![type_indicator as u8];
-    res.append(&mut size.to_le_bytes().to_vec());
-    res.append(&mut bytes);
-    return res;
+pub fn fast_from_bytes<T: Sized + AsBytes + FromBytes + Clone>(bytes: &[u8])
+ -> Result<T, ProgramError> {
+     let len = std::mem::size_of::<T>();
+     let mut buf = vec![0u8;len];
+     buf.copy_from_slice(&bytes[..len]);
+     let (res, _) = LayoutVerified::<&[u8], T>::new_from_prefix(&buf)
+         .ok_or(ProgramError::InvalidAccountData)?;
+     return Ok(res.clone());
 }
 
-pub fn borsh_safe_deserialize<T: BorshDeserialize>(bytes: &[u8]) -> Result<T, ProgramError> {
-    let size_buf: &[u8] = &bytes[1..9];
-    let size = usize::from_le_bytes(size_buf.try_into().unwrap());
-    let buf = &bytes[9..size + 9];
-    let res = &buf[..size].to_vec();
-    T::try_from_slice(res).map_err(|_| ProgramError::InvalidAccountData)
-}
+// pub fn borsh_safe_serialize<T: BorshSerialize>(state: &T, type_indicator: SwitchboardAccountType) -> Vec<u8> {
+    // msg!("borsh serialize");
+    // let mut bytes = state.try_to_vec().unwrap();
+    // let size: usize = bytes.len();
+    // let mut res = vec![type_indicator as u8];
+    // res.append(&mut size.to_le_bytes().to_vec());
+    // res.append(&mut bytes);
+    // return res;
+// }
+//
+// pub fn borsh_safe_deserialize<T: BorshDeserialize>(bytes: &[u8]) -> Result<T, ProgramError> {
+    // let size_buf: &[u8] = &bytes[1..9];
+    // let size = usize::from_le_bytes(size_buf.try_into().unwrap());
+    // let buf = &bytes[9..size + 9];
+    // let res = &buf[..size].to_vec();
+    // T::try_from_slice(res).map_err(|_| ProgramError::InvalidAccountData)
+// }
 
 pub fn save_state<T: MessageWrite>(
     account: &AccountInfo,
